"""Top-level package for xepmts."""

__author__ = """Yossi Mosbacher"""
__email__ = 'joe.mosbacher@gmail.com'
__version__ = '0.4.30'

from xepmts.db.client import default_client, get_client, get_admin_client
from . import streams

def settings(**kwargs):
    from eve_panel import settings as panel_settings
    if not kwargs:
        return dir(panel_settings)
    else:
        for k,v in kwargs.items():
            setattr(panel_settings, k, v)

def extension():
    import eve_panel
    eve_panel.extension()

notebook = extension


import httpx
import logging
import xepmts_endpoints
from xepmts_endpoints import endpoints


log = logging.getLogger(__name__)

def clean_nones(d):
    return {k:v for k,v in d.items() if v is not None}

def get_endpoints(servers, endpoint_path='endpoints'):
    for server in servers:
        uri = "/".join([server.rstrip('/'), endpoint_path.lstrip('/')])
        log.info(f"Attempting to read endpoints from {uri}")
        try:
            r = httpx.get(uri, timeout=5)
            if not r.is_error:
                log.info('Endpoints read succesfully from server.')
                return {k: clean_nones(v) for k,v in r.json().items()}
        except:
            pass
    log.error("Failed to read endpoint definitions from server, loading defaults.")
    return xepmts_endpoints.get_endpoints()


import time
import getpass
import os 

from .daq import LiveDAQStreamzViewer



def get_live_rate_viewer(db=None, api_user=None, api_key=None, detector="tpc"):
    if db is None:
        import xepmts
        db = xepmts.get_client("v2")
        if not db.logged_in:
            db.login()
            db.session.auth.authorize()
        while not db.logged_in:
            time.sleep(0.5)

    if not api_user:
        api_user = os.getenv("DAQ_API_USER", None)

    if not api_user:
        api_user = input("DAQ API user: ")

    if not api_key:
        api_key = os.getenv("DAQ_API_KEY", None)
    if api_key is None:
        api_key = getpass.getpass("DAQ API key: ")

    viewer = LiveDAQStreamzViewer(
        api_user=api_user,
        api_key=api_key,
        client=db,
    )

    viewer.add_stream(detector)
    return viewer.daq_streams[detector]


from .endpoints import get_endpoints
import os
import pkg_resources

DEFAULT_SERVER = "xenonnt.org"
SERVERS = {
    "xenonnt.org": "https://api.pmts.xenonnt.org/",
    "gae": "https://api-dot-xenon-pmts.uc.r.appspot.com/",
    "gae_proxy": "https://api-proxy-dot-xenon-pmts.uc.r.appspot.com/",
    "deta": "https://38nq2t.deta.dev/",
    "lngs": "https://xe1t-mysql.lngs.infn.it/api/",
}


def get_client(version, scopes=["read:all"], servers=None):
    import eve_panel
    if servers is None:
        servers = {f"{name}": f"{address.strip('/')}/{version}"
                    for name, address in SERVERS.items()}
        servers["default"] = f"{SERVERS[DEFAULT_SERVER].strip('/')}/{version}"
    elif isinstance(servers, str):
        servers = {'default': servers}
    elif isinstance(servers, (tuple,list)):
        servers = {f'server_{i}': server for i,server in enumerate(servers)}
    if not isinstance(servers, dict):
        raise ValueError("Servers parameter must be of type dict with signiture: {name: url}")

    endpoints = get_endpoints(servers.values())
    client = eve_panel.EveClient.from_domain_def(domain_def=endpoints, name="xepmts", auth_scheme="Bearer",
                             sort_by_url=True, servers=servers)
    if 'default' in servers:
        client.select_server("default")
    elif len(servers):
        client.select_server(list(servers)[0])
    client.db = client
    if version=="v2":
        client.set_auth("XenonAuth")
        client.set_credentials(audience="https://api.pmts.xenonnt.org", scopes=scopes)
        
    return client

def default_client():
    return get_client("v2")

def get_admin_client(servers=None):
    import eve_panel
    scopes = ['admin']
    version = 'admin'

    if servers is None:
        servers = {f"{name}": f"{address.strip('/')}/{version}"
                    for name, address in SERVERS.items()}
        servers["default"] = f"{SERVERS[DEFAULT_SERVER].strip('/')}/{version}"
    elif isinstance(servers, str):
        servers = {'default': servers}
    elif isinstance(servers, (tuple,list)):
        servers = {f'server_{i}': server for i,server in enumerate(servers)}
    if not isinstance(servers, dict):
        raise ValueError("Servers parameter must be of type dict with signiture: {name: url}")

    endpoints = get_endpoints(servers.values())
    client = eve_panel.EveClient.from_domain_def(domain_def=endpoints, name="xepmts", auth_scheme="Bearer",
                             sort_by_url=True, servers=servers)

    client.set_auth("XenonAuth")
    client.set_credentials(audience="https://api.pmts.xenonnt.org", scopes=scopes)
    return client